const { subtle } = require('node:crypto').webcrypto;
const stringutils = require("./stringutils");
const crypto = require('crypto');

const sharedSecret = process.env.SHARED_SECRET;

const authentication = {
    checkSharedSecret: (providedSecret) => {
        const sharedSecretBuffer = Buffer.from(sharedSecret);
        const providedSecretBuffer = Buffer.from(providedSecret);

        const length = Math.max(sharedSecretBuffer.length, providedSecretBuffer.length);
        const paddedSharedSecret = Buffer.alloc(length, 0);
        const paddedProvidedSecret = Buffer.alloc(length, 0);

        sharedSecretBuffer.copy(paddedSharedSecret);
        providedSecretBuffer.copy(paddedProvidedSecret);

        return crypto.timingSafeEqual(paddedSharedSecret, paddedProvidedSecret);
    },
    verifySignature : async (msg, sig, publicKeys) => {
        try {
            for (const hexKey of publicKeys) {
                try {
                    const pubKey = await stringutils.hexToKey(hexKey);
                    const verified = await subtle.verify(
                        'Ed25519',
                        pubKey,
                        sig,
                        msg
                    );
                    if (verified) {
                        return hexKey;
                    }
                } catch (err) {
                    console.log('Failed to verify signature with public key:', hexKey, err);
                }
            }
            return null;
        } catch (err) {
            console.error('Error verifying signature:', err);
        }
    }
};

module.exports = authentication;
